DrivenNet

Struct DrivenNet 

pub struct DrivenNet<I: Instantiable> { /* private fields */ }
Represent a net that is being driven by a Instantiable

Implementations

Source

impl<I> DrivenNet<I>
where I: Instantiable,

pub fn as_net(&self) -> Ref<'_, Net>

Borrow the net being driven

pub fn as_net_mut(&self) -> RefMut<'_, Net>

Get a mutable reference to the net being driven

pub fn is_an_input(&self) -> bool

Returns true if this net is a principal input

pub fn get_port(&self) -> Net

Get the output port associated with this connection

pub fn connect(&self, input: InputPort<I>)

Connects the net driven by this output port to the given input port.

pub fn is_top_level_output(&self) -> bool

Returns true if this net is a top-level output in the netlist.

pub fn unwrap(self) -> NetRef<I>

Return the underlying circuit node

pub fn get_identifier(&self) -> Identifier

Returns a copy of the identifier of the net being driven.

pub fn expose_with_name(self, name: Identifier) -> Self

Exposes this driven net as a top-level output with a specific port name. Multiple calls to this method can be used to create multiple output aliases for the same net.

Panics

Panics if the weak reference to the netlist is dead.

pub fn remove_output(&self, net_name: &Identifier) -> bool

Removes a specific output alias by its name from this driven net. Returns true if the output was removed, false if it didn’t exist.

Panics

Panics if the reference to the netlist is lost.

pub fn remove_all_outputs(&self) -> usize

Removes all output aliases for this driven net. Returns the number of outputs that were removed.

Panics

Panics if the reference to the netlist is lost.

pub fn get_output_index(&self) -> Option<usize>

Returns the output position, if the net is the output of a gate.

pub fn get_instance_type(&self) -> Option<Ref<'_, I>>

Returns the Instantiable type driving this net, if it has a driver.