Netlist

Struct Netlist 

pub struct Netlist<I>
where I: Instantiable,
{ /* private fields */ }
A netlist data structure

Implementations

Source

impl<I> Netlist<I>
where I: Instantiable,

pub fn new(name: String) -> Rc<Self>

Creates a new netlist with the given name

pub fn reclaim(self: Rc<Self>) -> Option<Self>

Attempts to reclaim the netlist, returning Some if successful.

pub fn deep_clone(self: &Rc<Self>) -> Rc<Self>

Creates a deep clone of the netlist.

pub fn insert_input(self: &Rc<Self>, net: Net) -> DrivenNet<I>

Inserts an input net to the netlist

pub fn insert_input_escaped_logic_bus( self: &Rc<Self>, net: String, bw: usize, ) -> Vec<DrivenNet<I>>

Inserts a four-state logic input port to the netlist

pub fn insert_gate( self: &Rc<Self>, inst_type: I, inst_name: Identifier, operands: &[DrivenNet<I>], ) -> Result<NetRef<I>, Error>

Inserts a gate to the netlist

pub fn insert_gate_disconnected( self: &Rc<Self>, inst_type: I, inst_name: Identifier, ) -> NetRef<I>

Use interior mutability to add an object to the netlist. Returns a mutable reference to the created object.

pub fn insert_constant( self: &Rc<Self>, value: Logic, inst_name: Identifier, ) -> Result<DrivenNet<I>, Error>

Inserts a constant Logic value to the netlist

pub fn get_driver(&self, netref: NetRef<I>, index: usize) -> Option<NetRef<I>>

Returns the driving node at input position index for netref

Panics

Panics if index is out of bounds The netref does not belong to this netlist

pub fn expose_net_with_name( &self, net: DrivenNet<I>, name: Identifier, ) -> DrivenNet<I>

Set an added object as a top-level output with a specific name. Multiple calls with different names for the same net will create multiple aliases.

Panics

The net does not belong to this netlist

pub fn expose_net(&self, net: DrivenNet<I>) -> Result<DrivenNet<I>, Error>

Sets the current net as a top-level output using the current name of the net

Panics

The net does not belong to this netlist

pub fn remove_output( &self, operand: &DrivenNet<I>, net_name: &Identifier, ) -> bool

Removes a specific output alias by its operand and net name. Returns true if the output was removed, false if it didn’t exist.

Panics

The operand does not belong to this netlist

pub fn remove_outputs(&self, operand: &DrivenNet<I>) -> usize

Removes all output aliases for a specific operand. Returns the number of outputs that were removed.

pub fn clear_outputs(&self)

Removes all outputs from the netlist.

pub fn delete_net_uses(&self, netref: NetRef<I>) -> Result<Object<I>, Error>

Unlink a circuit node from the rest of the netlist. Return the object that was being stored.

Panics

The netref does not belong to this netlist

pub fn replace_net_uses( &self, of: DrivenNet<I>, with: &DrivenNet<I>, ) -> Result<Object<I>, Error>

Replaces the uses of a circuit node with another circuit node. The Object stored at of is returned.

Panics

of or with do not belong to this netlist

Source

impl<I> Netlist<I>
where I: Instantiable,

pub fn get_name(&self) -> Ref<'_, String>

Returns the name of the netlist module

pub fn set_name(&self, name: String)

Sets the name of the netlist module

Panics

Panics if the module name cannot be borrowed mutably.

pub fn get_input_ports(&self) -> impl Iterator<Item = Net>

Iterates over the input ports of the netlist.

pub fn get_output_ports(&self) -> Vec<Net>

Returns a list of output nets

pub fn get_analysis<'a, A: Analysis<'a, I>>(&'a self) -> Result<A, Error>

Constructs an analysis of the netlist.

pub fn find_net(&self, net: &Net) -> Option<DrivenNet<I>>

Finds the first circuit node that drives the net. This operation is O(n). This should be unique provided the netlist is well-formed.

pub fn first(&self) -> Option<NetRef<I>>

Returns a NetRef to the first circuit node

pub fn last(&self) -> Option<NetRef<I>>

Returns a NetRef to the last circuit node

pub fn len(&self) -> usize

Returns the number of objects in the netlist (instances + inputs)

pub fn is_empty(&self) -> bool

Returns true if the netlist contains no objects.

pub fn drives_an_output(&self, netref: NetRef<I>) -> bool

Returns true if an output of netref which is driving a module output.

Panics

The netref does not belong to this netlist

pub fn rename_nets<F: Fn(&Identifier, usize) -> Identifier>( &self, f: F, ) -> Result<(), Error>

Rename nets and instances in the netlist using the provided injective function. Returns an error if the function is not injective.

Examples
use safety_net::format_id;
use safety_net::{Gate, GateNetlist};

let netlist = GateNetlist::new("example".to_string());
let inv = Gate::new_logical("INV".into(), vec!["A".into()], "Y".into());
let foo = netlist.insert_input("foo".into());
let nr = netlist.insert_gate(inv, "bar".into(), &[foo]).unwrap();
nr.expose_with_name("baz".into());
netlist.rename_nets(|id, i| format_id!("{}_{}", id, i) ).unwrap();
// "bar_Y" -> "bar_Y_0"
// "bar" -> "bar_1"
pub fn retain_once( &self, set: &mut HashSet<DrivenNet<I>>, ) -> Result<Vec<Object<I>>, Error>

Retains the DrivenNets in set, given they are used. Otherwise, they are cleaned and returned in a Ok(vec).

pub fn clean(&self) -> Result<Vec<Object<I>>, Error>

Removes unused nodes from the netlist, until it stops changing. Returns Ok(vec) of the removed objects.

pub fn retain( &self, set: &mut HashSet<DrivenNet<I>>, ) -> Result<Vec<Object<I>>, Error>

Retains the DrivenNets in set, given they are used. Otherwise, they are cleaned and returned in a Ok(vec).

pub fn verify(&self) -> Result<(), Error>

Verifies that a netlist is well-formed.

Source

impl<I> Netlist<I>
where I: Instantiable,

pub fn objects(&self) -> impl Iterator<Item = NetRef<I>>

Returns an iterator over the circuit nodes in the netlist.

pub fn matches<F>(&self, filter: F) -> impl Iterator<Item = NetRef<I>>
where F: Fn(&I) -> bool,

Returns an iterator over the circuit nodes that match the instance type.

pub fn inputs(&self) -> impl Iterator<Item = DrivenNet<I>>

Returns an iterator to principal inputs in the netlist as references.

pub fn outputs(&self) -> Vec<(DrivenNet<I>, Net)>

Returns an iterator to circuit nodes that drive an output in the netlist.

pub fn connections(&self) -> impl Iterator<Item = Connection<I>>

Returns an iterator over the wire connections in the netlist.

pub fn node_dfs(&self, from: NetRef<I>) -> impl Iterator<Item = NetRef<I>>

Returns a depth-first search iterator over the nodes in the netlist.

Panics

from does not belong to this netlist

pub fn net_dfs(&self, from: DrivenNet<I>) -> impl Iterator<Item = DrivenNet<I>>

Returns a depth-first search iterator over the nodes in the netlist, with the nodes in DrivenNet form.

Panics

from does not belong to this netlist