pub struct Netlist<I>where
I: Instantiable,{ /* private fields */ }Expand description
A netlist data structure
Implementations
Sourceimpl<I> Netlist<I>where
I: Instantiable,
impl<I> Netlist<I>where
I: Instantiable,
Sourcepub fn reclaim(self: Rc<Self>) -> Option<Self>
pub fn reclaim(self: Rc<Self>) -> Option<Self>
Attempts to reclaim the netlist, returning Some if successful.
Sourcepub fn deep_clone(self: &Rc<Self>) -> Rc<Self>
pub fn deep_clone(self: &Rc<Self>) -> Rc<Self>
Creates a deep clone of the netlist.
Sourcepub fn insert_input(self: &Rc<Self>, net: Net) -> DrivenNet<I>
pub fn insert_input(self: &Rc<Self>, net: Net) -> DrivenNet<I>
Inserts an input net to the netlist
Sourcepub fn insert_input_escaped_logic_bus(
self: &Rc<Self>,
net: String,
bw: usize,
) -> Vec<DrivenNet<I>>
pub fn insert_input_escaped_logic_bus( self: &Rc<Self>, net: String, bw: usize, ) -> Vec<DrivenNet<I>>
Inserts a four-state logic input port to the netlist
Sourcepub fn insert_gate(
self: &Rc<Self>,
inst_type: I,
inst_name: Identifier,
operands: &[DrivenNet<I>],
) -> Result<NetRef<I>, Error>
pub fn insert_gate( self: &Rc<Self>, inst_type: I, inst_name: Identifier, operands: &[DrivenNet<I>], ) -> Result<NetRef<I>, Error>
Inserts a gate to the netlist
Sourcepub fn insert_gate_disconnected(
self: &Rc<Self>,
inst_type: I,
inst_name: Identifier,
) -> NetRef<I>
pub fn insert_gate_disconnected( self: &Rc<Self>, inst_type: I, inst_name: Identifier, ) -> NetRef<I>
Use interior mutability to add an object to the netlist. Returns a mutable reference to the created object.
Sourcepub fn insert_constant(
self: &Rc<Self>,
value: Logic,
inst_name: Identifier,
) -> Result<DrivenNet<I>, Error>
pub fn insert_constant( self: &Rc<Self>, value: Logic, inst_name: Identifier, ) -> Result<DrivenNet<I>, Error>
Inserts a constant Logic value to the netlist
Sourcepub fn get_driver(&self, netref: NetRef<I>, index: usize) -> Option<NetRef<I>>
pub fn get_driver(&self, netref: NetRef<I>, index: usize) -> Option<NetRef<I>>
Returns the driving node at input position index for netref
Panics
Panics if index is out of bounds
The netref does not belong to this netlist
Sourcepub fn expose_net_with_name(
&self,
net: DrivenNet<I>,
name: Identifier,
) -> DrivenNet<I>
pub fn expose_net_with_name( &self, net: DrivenNet<I>, name: Identifier, ) -> DrivenNet<I>
Set an added object as a top-level output with a specific name. Multiple calls with different names for the same net will create multiple aliases.
Panics
The net does not belong to this netlist
Sourcepub fn expose_net(&self, net: DrivenNet<I>) -> Result<DrivenNet<I>, Error>
pub fn expose_net(&self, net: DrivenNet<I>) -> Result<DrivenNet<I>, Error>
Sets the current net as a top-level output using the current name of the net
Panics
The net does not belong to this netlist
Sourcepub fn remove_output(
&self,
operand: &DrivenNet<I>,
net_name: &Identifier,
) -> bool
pub fn remove_output( &self, operand: &DrivenNet<I>, net_name: &Identifier, ) -> bool
Removes a specific output alias by its operand and net name. Returns true if the output was removed, false if it didn’t exist.
Panics
The operand does not belong to this netlist
Sourcepub fn remove_outputs(&self, operand: &DrivenNet<I>) -> usize
pub fn remove_outputs(&self, operand: &DrivenNet<I>) -> usize
Removes all output aliases for a specific operand. Returns the number of outputs that were removed.
Sourcepub fn clear_outputs(&self)
pub fn clear_outputs(&self)
Removes all outputs from the netlist.
Sourceimpl<I> Netlist<I>where
I: Instantiable,
impl<I> Netlist<I>where
I: Instantiable,
Sourcepub fn get_input_ports(&self) -> impl Iterator<Item = Net>
pub fn get_input_ports(&self) -> impl Iterator<Item = Net>
Iterates over the input ports of the netlist.
Sourcepub fn get_output_ports(&self) -> Vec<Net>
pub fn get_output_ports(&self) -> Vec<Net>
Returns a list of output nets
Sourcepub fn get_analysis<'a, A: Analysis<'a, I>>(&'a self) -> Result<A, Error>
pub fn get_analysis<'a, A: Analysis<'a, I>>(&'a self) -> Result<A, Error>
Constructs an analysis of the netlist.
Sourcepub fn find_net(&self, net: &Net) -> Option<DrivenNet<I>>
pub fn find_net(&self, net: &Net) -> Option<DrivenNet<I>>
Finds the first circuit node that drives the net. This operation is O(n).
This should be unique provided the netlist is well-formed.
Sourcepub fn drives_an_output(&self, netref: NetRef<I>) -> bool
pub fn drives_an_output(&self, netref: NetRef<I>) -> bool
Returns true if an output of netref which is driving a module output.
Panics
The netref does not belong to this netlist
Sourcepub fn rename_nets<F: Fn(&Identifier, usize) -> Identifier>(
&self,
f: F,
) -> Result<(), Error>
pub fn rename_nets<F: Fn(&Identifier, usize) -> Identifier>( &self, f: F, ) -> Result<(), Error>
Rename nets and instances in the netlist using the provided injective function. Returns an error if the function is not injective.
Examples
use safety_net::format_id;
use safety_net::{Gate, GateNetlist};
let netlist = GateNetlist::new("example".to_string());
let inv = Gate::new_logical("INV".into(), vec!["A".into()], "Y".into());
let foo = netlist.insert_input("foo".into());
let nr = netlist.insert_gate(inv, "bar".into(), &[foo]).unwrap();
nr.expose_with_name("baz".into());
netlist.rename_nets(|id, i| format_id!("{}_{}", id, i) ).unwrap();
// "bar_Y" -> "bar_Y_0"
// "bar" -> "bar_1"Sourcepub fn retain_once(
&self,
set: &mut HashSet<DrivenNet<I>>,
) -> Result<Vec<Object<I>>, Error>
pub fn retain_once( &self, set: &mut HashSet<DrivenNet<I>>, ) -> Result<Vec<Object<I>>, Error>
Retains the DrivenNets in set, given they are used. Otherwise, they are cleaned and returned in a Ok(vec).
Sourcepub fn clean(&self) -> Result<Vec<Object<I>>, Error>
pub fn clean(&self) -> Result<Vec<Object<I>>, Error>
Removes unused nodes from the netlist, until it stops changing.
Returns Ok(vec) of the removed objects.
Sourceimpl<I> Netlist<I>where
I: Instantiable,
impl<I> Netlist<I>where
I: Instantiable,
Sourcepub fn objects(&self) -> impl Iterator<Item = NetRef<I>>
pub fn objects(&self) -> impl Iterator<Item = NetRef<I>>
Returns an iterator over the circuit nodes in the netlist.
Sourcepub fn matches<F>(&self, filter: F) -> impl Iterator<Item = NetRef<I>>
pub fn matches<F>(&self, filter: F) -> impl Iterator<Item = NetRef<I>>
Returns an iterator over the circuit nodes that match the instance type.
Sourcepub fn inputs(&self) -> impl Iterator<Item = DrivenNet<I>>
pub fn inputs(&self) -> impl Iterator<Item = DrivenNet<I>>
Returns an iterator to principal inputs in the netlist as references.
Sourcepub fn outputs(&self) -> Vec<(DrivenNet<I>, Net)>
pub fn outputs(&self) -> Vec<(DrivenNet<I>, Net)>
Returns an iterator to circuit nodes that drive an output in the netlist.
Sourcepub fn connections(&self) -> impl Iterator<Item = Connection<I>>
pub fn connections(&self) -> impl Iterator<Item = Connection<I>>
Returns an iterator over the wire connections in the netlist.